@@ -1,4 +1,4 @@
-/* $XdotOrg: xc/programs/Xserver/hw/xfree86/input/keyboard/kbd.c,v 1.16 2005/07/13 12:30:04 alanh Exp $ */
+/* $XdotOrg: driver/xf86-input-keyboard/src/kbd.c,v 1.19 2005/11/09 21:31:23 kem Exp $ */
 /* $XFree86: xc/programs/Xserver/hw/xfree86/input/keyboard/kbd.c,v 1.8 2003/11/03 05:11:47 tsi Exp $ */
 
 /*
@@ -12,11 +12,16 @@
  * xf86Events.c and xf86Io.c which are
  * Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
  */
-/* $XdotOrg: xc/programs/Xserver/hw/xfree86/input/keyboard/kbd.c,v 1.16 2005/07/13 12:30:04 alanh Exp $ */
+/* $XdotOrg: driver/xf86-input-keyboard/src/kbd.c,v 1.19 2005/11/09 21:31:23 kem Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+#include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
+#include <unistd.h>
   
 #define NEED_EVENTS
 #include <X11/X.h>
@@ -38,7 +43,6 @@
 #include "xf86Xinput.h"
 #include "xf86_OSproc.h"
 #include "xf86OSKbd.h"
-#include "xf86_ansic.h"
 #include "compiler.h"
 
 #ifdef XKB
@@ -110,7 +114,6 @@ typedef enum {
     OPTION_CUSTOM_KEYCODES
 } KeyboardOpts;
 
-#ifdef XFree86LOADER
 /* These aren't actually used ... */
 static const OptionInfoRec KeyboardOptions[] = {
     { OPTION_ALWAYS_CORE,	"AlwaysCore",	  OPTV_BOOLEAN,	{0}, FALSE },
@@ -136,10 +139,13 @@ static const OptionInfoRec KeyboardOptions[] = {
     { OPTION_CUSTOM_KEYCODES,   "CustomKeycodes", OPTV_BOOLEAN,	{0}, FALSE },
     { -1,			NULL,		  OPTV_NONE,	{0}, FALSE }
 };
-#endif
 
 static const char *kbdDefaults[] = {
+#ifdef XQUEUE 
+    "Protocol",		"Xqueue",
+#else
     "Protocol",		"standard",
+#endif
     "AutoRepeat",	"500 30",
     "XkbRules",		__XKBDEFRULES__,
     "XkbModel",		"pc105",
@@ -150,7 +156,11 @@ static const char *kbdDefaults[] = {
 };
 
 static const char *kbd98Defaults[] = {
+#ifdef XQUEUE
+    "Protocol",		"Xqueue",
+#else
     "Protocol",		"standard",
+#endif
     "AutoRepeat",	"500 30",
     "XkbRules",		"xfree98",
     "XkbModel",		"pc98",
@@ -176,14 +186,12 @@ static char *xkb_options;
 static XkbComponentNamesRec xkbnames;
 #endif /* XKB */
 
-#ifdef XFree86LOADER
 /*ARGSUSED*/
 static const OptionInfoRec *
 KeyboardAvailableOptions(void *unused)
 {
     return (KeyboardOptions);
 }
-#endif
 
 static void
 SetXkbOption(InputInfoPtr pInfo, char *name, char **option)
@@ -222,8 +230,6 @@ KbdPreInit(InputDriverPtr drv, IDevPtr dev, int flags)
      * an OS specific readInput() function to handle this.
      */
     pInfo->read_input = NULL;
-    pInfo->motion_history_proc = NULL;
-    pInfo->history_size = 0;
     pInfo->control_proc = NULL;
     pInfo->close_proc = NULL;
     pInfo->switch_mode = NULL;
@@ -377,7 +383,22 @@ KbdCtrl( DeviceIntPtr device, KeybdCtrl *ctrl)
    InputInfoPtr pInfo = (InputInfoPtr) device->public.devicePrivate;
    KbdDevPtr pKbd = (KbdDevPtr) pInfo->private;
 
-   if ( ctrl->leds & XCOMP ) {
+   if ( ctrl->leds & XLED1) {
+       pKbd->keyLeds |= CAPSFLAG;
+   } else {
+       pKbd->keyLeds &= ~CAPSFLAG;
+   }
+   if ( ctrl->leds & XLED2) {
+       pKbd->keyLeds |= NUMFLAG;
+   } else {
+       pKbd->keyLeds &= ~NUMFLAG;
+   }
+   if ( ctrl->leds & XLED3) {
+       pKbd->keyLeds |= SCROLLFLAG;
+   } else {
+       pKbd->keyLeds &= ~SCROLLFLAG;
+   }
+   if ( ctrl->leds & (XCOMP|XLED4) ) {
        pKbd->keyLeds |= COMPOSEFLAG;
    } else {
        pKbd->keyLeds &= ~COMPOSEFLAG;
@@ -413,6 +434,8 @@ InitKBD(InputInfoPtr pInfo, Bool init)
   kevent.u.keyButtonPointer.rootX = 0;
   kevent.u.keyButtonPointer.rootY = 0;
 
+/* The server does this for us with i-h. */
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 1
   /*
    * Hmm... here is the biggest hack of every time !
    * It may be possible that a switch-vt procedure has finished BEFORE
@@ -442,12 +465,21 @@ InitKBD(InputInfoPtr pInfo, Bool init)
           (* pKeyboard->public.processInputProc)(&kevent, pKeyboard, 1);
         }
       }
+#endif
+
   pKbd->scanPrefix      = 0;
 
   if (init) {
-      pKbd->keyLeds = 0;
-
+      pKbd->keyLeds = pKbd->GetLeds(pInfo);
       UpdateLeds(pInfo);
+      if (pKbd->keyLeds & CAPSFLAG) {
+	  pKbd->PostEvent(pInfo, KEY_CapsLock, TRUE);
+	  pKbd->PostEvent(pInfo, KEY_CapsLock, FALSE);
+      }
+      if (pKbd->keyLeds & NUMFLAG) {
+	  pKbd->PostEvent(pInfo, KEY_NumLock, TRUE);
+	  pKbd->PostEvent(pInfo, KEY_NumLock, FALSE);
+      }
 
       if( pKbd->delay <= 375) rad = 0x00;
       else if (pKbd->delay <= 625) rad = 0x20;
@@ -518,7 +550,6 @@ KbdProc(DeviceIntPtr device, int what)
      * passing on parts of the VT switch sequence.
      */
     if (pInfo->fd >= 0) {
-	sleep(1);
 	xf86FlushInput(pInfo->fd);
 	AddEnabledDevice(pInfo->fd);
     }
@@ -783,7 +814,6 @@ sunKeyboards:
    }
 }
 
-#ifdef XFree86LOADER
 ModuleInfoRec KbdInfo = {
     1,
     "KBD",
@@ -825,7 +855,7 @@ static XF86ModuleVersionInfo xf86KbdVersionRec =
     MODINFOSTRING1,
     MODINFOSTRING2,
     XORG_VERSION_CURRENT,
-    1, 0, 0,
+    PACKAGE_VERSION_MAJOR, PACKAGE_VERSION_MINOR, PACKAGE_VERSION_PATCHLEVEL,
     ABI_CLASS_XINPUT,
     ABI_XINPUT_VERSION,
     MOD_CLASS_XINPUT,
@@ -838,63 +868,3 @@ _X_EXPORT XF86ModuleData kbdModuleData = {
     xf86KbdPlug,
     xf86KbdUnplug
 };
-
-
-/* Compatibility section: we have a set of module structures here that
- * allows us to load this module as the old keyboard driver. */
-
-#ifndef USE_DEPRECATED_KEYBOARD_DRIVER
-
-ModuleInfoRec KeyboardInfo = {
-    1,
-    "KEYBOARD",
-    NULL,
-    0,
-    KeyboardAvailableOptions,
-};
-
-static pointer
-xf86KeyboardPlug(pointer	module,
-                 pointer	options,
-                 int		*errmaj,
-                 int		*errmin)
-{
-    static Bool Initialised = FALSE;
-
-    if (!Initialised) {
-	Initialised = TRUE;
-#ifndef REMOVE_LOADER_CHECK_MODULE_INFO
-	if (xf86LoaderCheckSymbol("xf86AddModuleInfo"))
-#endif
-	xf86AddModuleInfo(&KeyboardInfo, module);
-    }
-
-    xf86AddInputDriver(&KEYBOARD, module, 0);
-
-    return module;
-}
-
-static XF86ModuleVersionInfo xf86KeyboardVersionRec =
-{
-    "keyboard",
-    MODULEVENDORSTRING,
-    MODINFOSTRING1,
-    MODINFOSTRING2,
-    XORG_VERSION_CURRENT,
-    1, 0, 0,
-    ABI_CLASS_XINPUT,
-    ABI_XINPUT_VERSION,
-    MOD_CLASS_XINPUT,
-    {0, 0, 0, 0}		/* signature, to be patched into the file by */
-				/* a tool */
-};
-
-_X_EXPORT XF86ModuleData keyboardModuleData = {
-    &xf86KeyboardVersionRec,
-    xf86KeyboardPlug,
-    xf86KbdUnplug
-};
-
-#endif /* ! USE_DEPRECATED_KEYBOARD_DRIVER */
-
-#endif /* XFree86LOADER */
